{% extends 'admin/base_site.html' %}
{% load i18n static admin_urls admin_modify %}

{% block coltype %}{% endblock %}

{% block title %}{{ object }}{% endblock %}

{% if not is_popup %}
  {% block breadcrumbs %}
  <div class="breadcrumbs">
  <a href="{% url 'admin:index' %}">{% translate 'Home' %}</a>
    &rsaquo; <a href="{% url opts|admin_urlname:'changelist' %}">{{ opts.verbose_name_plural|capfirst }}</a>
    &rsaquo; {{ object }}
  </div>
  {% endblock %}
{% endif %}

{% block content %}
  <div id="content-main">
    
    {% block object-tools %}
        <ul class="object-tools">
          {% block object-tools-items %}
            {% include 'admin/inspect_object_tools.html' %}
          {% endblock %}
        </ul>
    {% endblock %}
    
    {% block content_main %}
        {% trans "Inspect" %} {{ object }}
    {% endblock %}

  </div>
{% endblock %}
